<?php

namespace plugin\points_mall\service;

use Exception;
use plugin\points_mall\model\PointsMallOrder;
use plugin\points_mall\model\PointsMallOrderItem;
use plugin\points_mall\model\PointsMallOrderSender;
use think\admin\Library;

class UserOrderService
{
    /**
     * 同步订单关联商品的库存
     * @param string $orderNo 订单编号
     * @return boolean
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\DbException
     * @throws \think\db\exception\ModelNotFoundException
     */
    public static function stock(string $orderNo): bool
    {
        $map = ['order_no' => $orderNo];
        $codes = PointsMallOrderItem::mk()->where($map)->column('gcode');
        foreach (array_unique($codes) as $code) GoodsService::stock($code);
        return true;
    }

    /**
     * 获取订单模型
     * @param PointsMallOrder|string $order
     * @param ?integer $unid 动态绑定变量
     * @param ?string $orderNo 动态绑定变量
     * @return PointsMallOrder
     * @throws \think\admin\Exception
     */
    public static function widthOrder($order, ?int &$unid = 0, ?string &$orderNo = '')
    {
        if (is_string($order)) {
            $order = PointsMallOrder::mk()->where(['order_no' => $order])->findOrEmpty();
        }
        if ($order instanceof PointsMallOrder) {
            [$unid, $orderNo] = [intval($order->getAttr('unid')), $order->getAttr('order_no')];
            return $order;
        }
        throw new Exception("无效订单对象！");
    }

    /**
     * 更新订单支付状态
     * @param PluginWemallOrder|string $order 订单模型
     * @param PluginPaymentRecord $payment 支付行为记录
     * @return array|bool|string|void|null
     * @throws \think\admin\Exception
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\DbException
     * @throws \think\db\exception\ModelNotFoundException
     * @remark 订单状态(0已取消,1预订单,2待支付,3待审核,4待发货,5已发货,6已收货,7已评论)
     */
    public static function change($order, PluginPaymentRecord $payment)
    {
        $order = self::widthOrder($order);
        if ($order->isEmpty()) return null;

        // 同步订单支付统计
        $ptotal = Payment::totalPaymentAmount($payment->getAttr('order_no'));
        $order->appendData([
            'payment_time'    => $payment->getAttr('create_time'),
            'payment_amount'  => $ptotal['amount'] ?? 0,
            'amount_payment'  => $ptotal['payment'] ?? 0,
            'amount_balance'  => $ptotal['balance'] ?? 0,
            'amount_integral' => $ptotal['integral'] ?? 0,
        ], true);

        // 订单已经支付完成
        if ($order->getAttr('payment_amount') >= $order->getAttr('amount_real')) {
            // 已完成支付，更新订单状态
            $status = $order->getAttr('delivery_type') ? 4 : 5;
            $order->save(['status' => $status, 'payment_status' => 1]);
            // 确认完成支付，发放余额积分奖励及升级返佣
        }

        // 退款或部分退款，仅更新订单支付统计
        if ($payment->getAttr('refund_status')) {
            return $order->save();
        }

        // 提交支付凭证，只需更新订单状态为【待审核】
        $isVoucher = $payment->getAttr('channel_type') === Payment::VOUCHER;
        if ($isVoucher && $payment->getAttr('audit_status') === 1) {
            return $order->save(['status' => 3, 'payment_status' => 1]);
        }

        // 凭证支付审核被拒绝，订单回滚到未支付状态
        if ($isVoucher && $payment->getAttr('audit_status') === 0) {
            if ($order->getAttr('status') === 3) $order->save(['status' => 2]);
        } else {
            $order->save();
        }
    }

    /**
     * 更新订单收货地址
     * @param PointsMallOrder $order
     * @param $address
     * @return boolean
     * @throws \think\admin\Exception
     */
    public static function perfect(PointsMallOrder $order, $address): bool
    {
        $unid = $order->getAttr('unid');
        $orderNo = $order->getAttr('order_no');
        // 创建订单发货信息
        $data = [
            'delivery_code'   => 0, 'delivery_count' => 0, 'unid' => $unid,
            'delivery_remark' => '', 'delivery_amount' => 0, 'status' => 1,
        ];
        $data['order_no'] = $orderNo;
        $data['address_id'] = $address->getAttr('id');
        // 收货人信息
        $data['user_name'] = $address->getAttr('user_name');
        $data['user_phone'] = $address->getAttr('user_phone');
        $data['user_idcode'] = $address->getAttr('idcode');
        $data['user_idimg1'] = $address->getAttr('idimg1');
        $data['user_idimg2'] = $address->getAttr('idimg2');
        // 收货地址信息
        $data['region_prov'] = $address->getAttr('region_prov');
        $data['region_city'] = $address->getAttr('region_city');
        $data['region_area'] = $address->getAttr('region_area');
        $data['region_addr'] = $address->getAttr('region_addr');
        // 记录原地址信息
        $data['extra'] = $data;
        PointsMallOrderSender::mk()->where(['order_no' => $orderNo])->findOrEmpty()->save($data);
        // 组装更新订单数据, 重新计算订单金额
        $update = ['status' => 2, 'amount_express' => $data['delivery_amount']];
        $update['amount_real'] = round($order->getAttr('amount_discount') - $order->getAttr('amount_reduct'), 2);
        $update['amount_total'] = round($order->getAttr('amount_goods'), 2);
        // 支付金额不能少于零
        if ($update['amount_real'] <= 0) $update['amount_real'] = 0.00;
        if ($update['amount_total'] <= 0) $update['amount_total'] = 0.00;
        // 更新用户订单数据
        if ($order->save($update)) {
            // 触发订单确认事件
            Library::$sapp->event->trigger('PluginWemallOrderPerfect', $order);
            // 返回处理成功数据
            return true;
        } else {
            return false;
        }
    }

}